from cheno import MannWhitney, Wilcoxon, KruskalWallis, Friedman
from scipy import stats
from math import sqrt, isclose

# Mann-Whitney Unit tests
def test_mann_whitney_andy_fields_correctness_without_data_test_p_value():
    p = 0.000493
    r = -0.78
    m = MannWhitney()
    ranks = m.solve(p, r, [None, None], tails=2)
    s1, s2 = sum(ranks[0]), sum(ranks[1])
    w = s1 if s1 < s2 else s2
    n1, n2 = len(ranks[0]), len(ranks[1])
    w_mean = n1 * (n1 + n2 + 1) / 2
    se_w = sqrt(n1 * n2 * (n1 + n2 + 1) / 12)
    z = (w - w_mean) / se_w
    p_prime = stats.norm.cdf(z) * 2
    print((p-p_prime)/p)
    assert isclose(p, p_prime, rel_tol=1e-3)


def test_mann_whitney_paul_denny_correctness_without_data_test_p_value():
    p = 0.001
    r = -0.214589885
    m = MannWhitney()
    ranks = m.solve(p, r, [163, None], tails=2)
    s1, s2 = sum(ranks[0]), sum(ranks[1])
    w = s1 if s1 < s2 else s2
    n1, n2 = len(ranks[0]), len(ranks[1])
    w_mean = min(n1, n2) * (n1 + n2 + 1) / 2
    se_w = sqrt(n1 * n2 * (n1 + n2 + 1) / 12)
    z = (w - w_mean) / se_w
    p_prime = stats.norm.cdf(z) * 2
    assert isclose(p, p_prime, abs_tol=.01)


# Wilcoxon Signed Ranks Unit tests
def test_wilcoxon_andy_fields_correctness_without_data_test_p_value():
    p = .047
    r = -0.44
    w = Wilcoxon()
    ranks = w.solve(p, r, tails=2)
    s1, s2 = sum(ranks[0]), sum(ranks[1])
    t = s1 if s1 < s2 else s2
    n = len(ranks[0]) + len(ranks[1])
    t_mean = n * (n + 1) / 4
    se_t = sqrt(n * (n + 1) * (2 * n + 1) / 24)
    z = (t - t_mean) / se_t
    p_prime = stats.norm.cdf(z) * 2
    assert isclose(p, p_prime, abs_tol=.01)


# Kruskal-Wallis H-test Unit tests
def test_kruskal_wallis_andy_fields_correctness_without_data_test_p_value():
    p = .034186
    eta2 = .07349
    kw = KruskalWallis(*[None, None, None, None])
    ranks, data = kw.solve(p, eta2)
    s = [(sum(x), len(x)) for x in ranks]
    n = sum([x[1] for x in s])
    h = (12 / (n * (n + 1))) * sum([x[0] ** 2 / x[1] for x in s]) - 3 * (n + 1)
    p_prime = stats.chi2.sf(h, 3)
    print(p_prime)
    assert isclose(p, p_prime, abs_tol=.01)


# Friedman Analysis Unit tests
def test_friedman_andy_fields_correctness_without_data_test_p_value():
    p = .905
    w = .00333
    k = 3
    f = Friedman()
    ranks = f.solve(p, w, k)
    s = [(sum(x), len(x)) for x in ranks.T]
    n = sum([x[1] for x in s]) / k
    h = (12 / (n * k * (k + 1))) * sum([x[0] ** 2 for x in s]) - 3 * n * (k + 1)
    p_prime = stats.chi2.sf(h, k - 1)
    assert isclose(p, p_prime, abs_tol=.01)


if __name__ == '__main__':
    # test_mann_whitney_andy_fields_correctness_without_data_test_p_value()
    # test_mann_whitney_paul_denny_correctness_without_data_test_p_value()
    # test_mann_whitney_generated_datasets()
    # test_wilcoxon_andy_fields_correctness_without_data_test_p_value()
    test_kruskal_wallis_andy_fields_correctness_without_data_test_p_value()
    # test_friedman_andy_fields_correctness_without_data_test_p_value()
