from cheno import StudentD, StudentI, ANOVA
from math import sqrt, isclose
from scipy import stats


def test_student_d_andy_fields_correctness_without_data_test_p_value():
    p = .031
    d = -0.7138936078529723
    s = StudentD()
    mean, s, n, _ = s.solve(p, d, tails=2)
    _, p_prime = _ttest_from_stats(mean, s, n)
    assert isclose(p, p_prime, rel_tol=0.001)


def test_student_ind_eq_n_andy_fields_correctness_without_data_test_p_value():
    p = .107
    d = -0.6862653762697537
    s = StudentI()
    means, ss, ns, data = s.solve(p, d, tails=2, eq_n=True)
    _, p_prime = stats.ttest_ind(*data)
    assert isclose(p, p_prime, rel_tol=0.001)


def test_anova_eq_n_andy_fields_correctness_without_data_test_p_value():
    p = 0.025
    eta2 = .46
    a = ANOVA(k=3)
    means, ss, ns, data = a.solve(p, eta2, g_mean=3.467)
    _, p_prime = stats.f_oneway(*data)
    assert isclose(p, p_prime, rel_tol=0.01)


def _ttest_from_stats(mean, s, n):
    t = mean / (s / sqrt(n))
    p = stats.t.sf(abs(t), n - 1) * 2
    return t, p


if __name__ == "__main__":
    # test_student_d_andy_fields_correctness_without_data_test_p_value()
    test_student_ind_eq_n_andy_fields_correctness_without_data_test_p_value()
    test_anova_eq_n_andy_fields_correctness_without_data_test_p_value()
